/* Tree Widget
 * Copyright (C) 1999 - Jeff Heflin - University of Maryland
 *
 * This source code may be redistributed or modified provided that
 * this notice is unchanged and all modifications are clearly indicated.
 * No portion of this code may be included in a commercial product without
 * the written permission of the author.
 *
 * The software and related materials are provided "as-is" without
 * warranty or indemnity of any kind, including any warranties of
 * use, performance or merchantability or fitness for a particular use
 * or purpose or for any purpose whatsoever. User bears all risk
 * relating to use, quality and performance of the software.
 *
 * Author: Jeff Heflin
 * e-mail: heflin@cs.umd.edu
 *
 */

package UserInt;

import java.awt.*;
import java.util.*;
import Tree.*;
import java.lang.System;

/** A user interface component for displaying tree structures
  from which the user can make selections. */
public class TreeWidget extends ObjectList {

  Vector treeList;              // vector of TreeModels
  int    toggleClickCount = 1;  // number of clicks before a node is expanded

  public TreeWidget(int rows, boolean multiple) {

    super(rows, multiple);   
  }

  /** Constructs a new tree widget with the given number of rows where
    only a single item can selected at any given time. */
  public TreeWidget(int rows) {

    super(rows, false);
  }

  /** Initiliaze the tree by giving it a vector of tree models consisting
    of labeled tree nodes. */
  public void initTree(Vector treeList) {

    DefaultTreeModel tree;
    LabeledTreeNode  topNode;
    int              i;

    this.treeList = treeList;    

    clear();         // overwrite any existing items in the list

    // add the top-level nodes to the visible list
    for (i=0; i < treeList.size(); i++) {
      tree = (DefaultTreeModel)treeList.elementAt(i);
      topNode = (LabeledTreeNode)tree.getRoot();
      addItem(topNode, 0);
    }
  }

  /** Initialize the tree by giving it a single tree model. */
  public void initTree(DefaultTreeModel tree) {

    Vector treeVec;

    treeVec = new Vector();
    treeVec.addElement(tree);
    initTree(treeVec);
  }

  public void addItem(LabeledTreeNode node, int level) {
    
    int i;
    String label;

    TreeItem ti = new TreeItem(node, level);
    label = node.getLabel();
    for (i=0; i<level; i++)
      label = "  " + label;
    super.addItem(label, ti);
  }     

  public void addItem(LabeledTreeNode node, int level, int index) {

    int i;
    String label;

    TreeItem ti = new TreeItem(node, level);
    label = node.getLabel();
    for (i=0; i<level; i++)
      label = "  "+label;
    super.addItem(label, ti, index);
  }   

  public void expand(int index){

    TreeItem ti;
    int level;
    int nindex;
    int i;
    LabeledTreeNode item;
    Enumeration     childEnum;    
    LabeledTreeNode child;

    ti=(TreeItem)super.getObject(index);
    if (ti.isExpand==true)
      return;
    ti.isExpand=true;
    level=ti.level+1;
    nindex=index+1;
	
    item=(LabeledTreeNode)ti.node;
    childEnum = item.children();
    while (childEnum.hasMoreElements()) {
      child = (LabeledTreeNode)childEnum.nextElement();
      addItem(child, level, nindex);
      nindex++;
    }
  }

  /** Expands the entire tree. */
  public void expandAll() {

    int i = 0;
    while (i < countItems()) {
      expand(i);
      i++;
    }
  }

  public void compact(int index){

    TreeItem ti;
    int level;
    int nindex;

    
    ti = (TreeItem)super.getObject(index);
    if (ti.isExpand==false)
      return;
    ti.isExpand=false;
    level=ti.level;
    nindex=index+1;
    
    while (true) {
      if (nindex>=countItems())
	break; 
      ti=(TreeItem)super.getObject(nindex);
      if (ti==null)
	break;
      if (ti.level<=level)
	break;
      delItem(nindex);
    }
  }


  public void changeState(int index){

    TreeItem ti;
 
    ti=(TreeItem)super.getObject(index);
    if (ti.isExpand==false)
      expand(index);
    else
      compact(index);
  }

  /** Returns the node that has been selected by the user. */
  public LabeledTreeNode getSelectedNode() {

    LabeledTreeNode node;

    TreeItem ti=(TreeItem)super.getSelectedObject();
    if (ti==null)
      return null;
    node = (LabeledTreeNode)ti.node;
    return node;
  }

  /** Returns the label of the node that was selected by the user. If
   no items were selected, returns null. */
  public String getSelectedLabel() {

    LabeledTreeNode node;

    node = getSelectedNode();
    if (node == null)
      return null;
    else
      return node.getLabel();
  }

  /** Returns the object associated with the node that was selected by
    the user. If no items were selected, returns null. */
  public Object getSelectedObject() {

    LabeledTreeNode node;

    node = getSelectedNode();
    if (node == null)
      return null;
    else
      return node.getUserObject();
  }


  public Object getObject(int index) {

    LabeledTreeNode node;

    TreeItem ti=(TreeItem)super.getObject(index);   
    if (ti==null)
      return null;
    node = (LabeledTreeNode)ti.node;
    return node.getUserObject();
  }


  public boolean handleEvent(Event evt) {   
   // System.out.println(evt.id+"  "+evt.clickCount);   
 
    if (toggleClickCount == 1 && evt.id==Event.LIST_SELECT){
      int index;
      // System.out.println("mousedown "+evt.clickCount);    
      index = getSelectedIndex();
      changeState(index);  
    }

    return super.handleEvent(evt);
  }

  public boolean action(Event evt, Object arg) {

    if (toggleClickCount == 2 && evt.target == this) {
      int index;
      // System.out.println("mousedown "+evt.clickCount);    
      index = getSelectedIndex();
      changeState(index);  
      return true;
    }
    return false;
  }

  /** Sets the number of clicks before a selected node's expansion
    status is toggled. Typically 1 or 2. */
  public void setToggleClickCount(int toggleClickCount) {

    this.toggleClickCount = toggleClickCount;
  }
}

/** A displayed item for the TreeWidget component. */
class TreeItem {

  boolean isExpand;
  int level;
  LabeledTreeNode node;
  
  public TreeItem(LabeledTreeNode node, int level) {

    this.level = level;
    this.node = node;
    isExpand=false;
  }

}








